@@ -1,4 +1,3 @@
-# -*- coding: utf8 -*-
 import asyncio
 import errno
 import fcntl
@@ -34,6 +33,20 @@ RAND_BYTES = os.urandom(10)
 tempdir = Path(tempfile.gettempdir()).resolve()
 IS_MACOS = platform.system() in ("AIX", "Darwin")
 
+SIGNALS_THAT_SHOULD_THROW_EXCEPTION = [
+    signal.SIGABRT,
+    signal.SIGBUS,
+    signal.SIGFPE,
+    signal.SIGILL,
+    signal.SIGINT,
+    signal.SIGKILL,
+    signal.SIGPIPE,
+    signal.SIGQUIT,
+    signal.SIGSEGV,
+    signal.SIGTERM,
+    signal.SIGSYS,
+]
+
 
 def hash(a: str):
     h = md5(a.encode("utf8") + RAND_BYTES)
@@ -88,6 +101,7 @@ baked_env = os.environ.copy()
 append_module_path(baked_env, sh)
 python = system_python.bake(_env=baked_env, _return_cmd=True)
 pythons = python.bake(_return_cmd=False)
+python_bg = system_python.bake(_env=baked_env, _bg=True)
 
 
 def requires_progs(*progs):
@@ -446,7 +460,7 @@ while True:
         def inc(*args, **kwargs):
             return python("-u", inc_py.name, *args, **kwargs)
 
-        class Derp(object):
+        class Derp:
             def __init__(self):
                 self.times = []
                 self.stdout = []
@@ -795,7 +809,7 @@ exit(2)
             self.assertEqual(gcc._path, gcc_file2)
             self.assertEqual(
                 gcc("no-error", _return_cmd=True).stdout.strip(),
-                "no-error".encode("ascii"),
+                b"no-error",
             )
 
         finally:
@@ -1708,7 +1722,6 @@ print("hello")
         )
 
         alternating = []
-        q = AQueue()
 
         async def producer(q):
             alternating.append(1)
@@ -1723,19 +1736,20 @@ print("hello")
             self.assertEqual(msg, "hello")
             alternating.append(2)
 
-        loop = asyncio.get_event_loop()
-        fut = asyncio.gather(producer(q), consumer(q))
-        loop.run_until_complete(fut)
+        async def main():
+            q = AQueue()
+            await asyncio.gather(producer(q), consumer(q))
+
+        asyncio.run(main())
         self.assertListEqual(alternating, [1, 2, 1, 2])
 
     def test_async_exc(self):
         py = create_tmp_test("""exit(34)""")
 
         async def producer():
-            await python(py.name, _async=True)
+            await python(py.name, _async=True, _return_cmd=False)
 
-        loop = asyncio.get_event_loop()
-        self.assertRaises(sh.ErrorReturnCode_34, loop.run_until_complete, producer())
+        self.assertRaises(sh.ErrorReturnCode_34, asyncio.run, producer())
 
     def test_async_iter(self):
         py = create_tmp_test(
@@ -1744,7 +1758,6 @@ for i in range(5):
     print(i)
 """
         )
-        q = AQueue()
 
         # this list will prove that our coroutines are yielding to eachother as each
         # line is produced
@@ -1764,9 +1777,11 @@ for i in range(5):
                     return
                 alternating.append(2)
 
-        loop = asyncio.get_event_loop()
-        res = asyncio.gather(producer(q), consumer(q))
-        loop.run_until_complete(res)
+        async def main():
+            q = AQueue()
+            await asyncio.gather(producer(q), consumer(q))
+
+        asyncio.run(main())
         self.assertListEqual(alternating, [1, 2, 1, 2, 1, 2, 1, 2, 1, 2])
 
     def test_async_iter_exc(self):
@@ -1784,8 +1799,36 @@ exit(34)
             async for line in python(py.name, _async=True):
                 lines.append(int(line.strip()))
 
-        loop = asyncio.get_event_loop()
-        self.assertRaises(sh.ErrorReturnCode_34, loop.run_until_complete, producer())
+        self.assertRaises(sh.ErrorReturnCode_34, asyncio.run, producer())
+
+    def test_async_return_cmd(self):
+        py = create_tmp_test(
+            """
+import sys
+sys.exit(0)
+"""
+        )
+
+        async def main():
+            result = await python(py.name, _async=True, _return_cmd=True)
+            self.assertIsInstance(result, sh.RunningCommand)
+            result_str = await python(py.name, _async=True, _return_cmd=False)
+            self.assertIsInstance(result_str, str)
+
+        asyncio.run(main())
+
+    def test_async_return_cmd_exc(self):
+        py = create_tmp_test(
+            """
+import sys
+sys.exit(1)
+"""
+        )
+
+        async def main():
+            await python(py.name, _async=True, _return_cmd=True)
+
+        self.assertRaises(sh.ErrorReturnCode_1, asyncio.run, main())
 
     def test_handle_both_out_and_err(self):
         py = create_tmp_test(
@@ -2308,7 +2351,7 @@ sys.stdout.write("line1")
 """
         )
 
-        class Callable(object):
+        class Callable:
             def __init__(self):
                 self.line = None
 
@@ -2647,7 +2690,7 @@ p.wait()
     def test_done_callback(self):
         import time
 
-        class Callback(object):
+        class Callback:
             def __init__(self):
                 self.called = False
                 self.exit_code = None
@@ -2792,7 +2835,7 @@ time.sleep(0.5)
     def test_done_cb_exc(self):
         from sh import ErrorReturnCode
 
-        class Callback(object):
+        class Callback:
             def __init__(self):
                 self.called = False
                 self.success = None
@@ -3109,6 +3152,61 @@ for line in sys.stdin:
             ErrorReturnCode_2, python, middleman_normal_pipe, consumer.name
         )
 
+    def test_bad_sig_raise_exception(self):
+        # test all bad signal are correctly raised
+        py = create_tmp_test(
+            """
+import time
+import sys
+
+time.sleep(2)
+sys.exit(1)
+"""
+        )
+        for sig in SIGNALS_THAT_SHOULD_THROW_EXCEPTION:
+            if sig == signal.SIGPIPE:
+                continue
+            sig_exception_name = f"SignalException_{sig}"
+            sig_exception = getattr(sh, sig_exception_name)
+            try:
+                p = python_bg(py.name)
+                time.sleep(0.5)
+                p.signal(sig)
+                p.wait()
+            except sig_exception:
+                pass
+            else:
+                self.fail(f"{sig_exception_name} not raised")
+
+    def test_ok_code_ignores_bad_sig_exception(self):
+        # Test if I have [-sig] in _ok_code, the exception won't be raised
+        py = create_tmp_test(
+            """
+import time
+import sys
+
+time.sleep(2)
+sys.exit(1)
+"""
+        )
+        for sig in SIGNALS_THAT_SHOULD_THROW_EXCEPTION:
+            if sig == signal.SIGPIPE:
+                continue
+            sig_exception_name = f"SignalException_{sig}"
+            sig_exception = getattr(sh, sig_exception_name)
+            python_bg_no_sig_exception = python_bg.bake(_ok_code=[-sig])
+            try:
+                p = python_bg_no_sig_exception(py.name)
+                time.sleep(0.5)
+                p.signal(sig)
+                p.wait()
+            except sig_exception:
+                self.fail(
+                    f"{sig_exception_name} should not be raised setting _ok_code."
+                )
+            else:
+                self.assertEqual(p.exit_code, -sig)
+
 
 class MockTests(BaseTests):
     def test_patch_command_cls(self):
